import pytest

from .type_id_registry import TypeIdREgistryMixin


def test_registry():
    class A(TypeIdREgistryMixin):
        _type_id_registry = {}
    class B(A):
        pass
    class C(A):
        pass

    # pylint: disable=protected-access
    assert A._type_id_registry['B'] == B
    assert A._type_id_registry['C'] == C


def test_build_by_type_id():
    class D(TypeIdREgistryMixin):
        _type_id_registry = {}
    built = []
    class E(D):
        def __init__(self):
            built.append(self)
    class F(D):
        def __init__(self):
            built.append(self)
            raise RuntimeError('cica')
    class H(E):
        pass

    D.build_type('E')
    assert isinstance(built[0], E)

    with pytest.raises(RuntimeError) as err:
        D.build_type('F')
    assert err.value.args[0] == 'cica'
    assert isinstance(built[1], F)

    D.build_type('H')
    assert isinstance(built[2], H)
    assert isinstance(built[2], E)
