from abc import ABCMeta
from contextlib import suppress


class TypeIdREgistryMeta(ABCMeta):
    def __init__(cls, name, bases, attrs):
        with suppress(AttributeError):
            if cls.__name__ in cls._type_id_registry:
                raise RuntimeError('Type id must be unique!')
            cls._type_id_registry[cls.__name__] = cls
        super().__init__(name, bases, attrs)


class TypeIdREgistryMixin(metaclass=TypeIdREgistryMeta):
    @classmethod
    def build_type(cls, type_id, *args, **kwargs):
        try:
            instance_type = cls._type_id_registry[type_id]
            return instance_type(*args, **kwargs)
        except KeyError:
            raise RuntimeError(f'No type with id {type_id}!')
